@@ -20,7 +20,6 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
-from awlsim.core.compat import *
 
 import sys
 
@@ -58,6 +57,11 @@ if isPy2Compat:
 if isPy2Compat:
 	range = xrange
 
+# reduce() compatibility.
+# Force Python2 behavior
+if isPy3Compat:
+	from functools import reduce
+
 # Compat wrapper for monotonic time
 import time
 monotonic_time = getattr(time, "monotonic", time.clock)
